@@ -1,5 +1,5 @@
-# Copyright (C) 2009-2013  Roman Zimbelmann <hut@lavabit.com>
-# This software is distributed under the terms of the GNU GPL version 3.
+# This file is part of ranger, the console file manager.
+# License: GNU GPL version 3, see the file "AUTHORS" for details.
 
 # TODO: Add an optional "!" to all commands and set a flag if it's there
 
@@ -11,10 +11,7 @@ from ranger.api import *
 from ranger.core.shared import FileManagerAware
 from ranger.ext.lazy_property import lazy_property
 
-SETTINGS_RE = re.compile(r'^\s*([^\s]+?)=(.*)$')
-DELETE_WARNING = 'delete seriously? '  # COMPAT
-
-def alias(*_): pass # COMPAT
+_SETTINGS_RE = re.compile(r'^\s*([^\s]+?)=(.*)$')
 
 class CommandContainer(object):
     def __init__(self):
@@ -75,7 +72,7 @@ class CommandContainer(object):
                 return None
 
     def command_generator(self, start):
-        return (cmd + ' ' for cmd in self.commands if cmd.startswith(start))
+        return sorted(cmd + ' ' for cmd in self.commands if cmd.startswith(start))
 
 
 class Command(FileManagerAware):
@@ -92,6 +89,7 @@ class Command(FileManagerAware):
         self.line = line
         self.args = line.split()
         self.quantifier = quantifier
+        self.quickly_executed = False
         try:
             self.firstpart = line[:line.rindex(' ') + 1]
         except ValueError:
@@ -155,7 +153,7 @@ class Command(FileManagerAware):
     def parse_setting_line(self):
         if self._setting_line is not None:
             return self._setting_line
-        match = SETTINGS_RE.match(self.rest(1))
+        match = _SETTINGS_RE.match(self.rest(1))
         if match:
             self.firstpart += match.group(1) + '='
             result = [match.group(1), match.group(2), True]
@@ -164,6 +162,38 @@ class Command(FileManagerAware):
         self._setting_line = result
         return result
 
+    def parse_flags(self):
+        """Finds and returns flags in the command
+
+        >>> Command("").parse_flags()
+        ('', '')
+        >>> Command("foo").parse_flags()
+        ('', '')
+        >>> Command("shell test").parse_flags()
+        ('', 'test')
+        >>> Command("shell -t ls -l").parse_flags()
+        ('t', 'ls -l')
+        >>> Command("shell -f -- -q test").parse_flags()
+        ('f', '-q test')
+        >>> Command("shell -foo -bar rest of the command").parse_flags()
+        ('foobar', 'rest of the command')
+        """
+        flags = ""
+        args = self.line.split()
+        rest = ""
+        if len(args) > 0:
+            rest = self.line[len(args[0]):].lstrip()
+            for arg in args[1:]:
+                if arg == "--":
+                    rest = rest[2:].lstrip()
+                    break
+                elif len(arg) > 1 and arg[0] == "-":
+                    rest = rest[len(arg):].lstrip()
+                    flags += arg[1:]
+                else:
+                    break
+        return flags, rest
+
     # XXX: Lazy properties? Not so smart? self.line can change after all!
     @lazy_property
     def _tabinsert_left(self):
@@ -241,22 +271,41 @@ class Command(FileManagerAware):
         rel_dirname = dirname(rel_dest)
 
         try:
+            directory = self.fm.get_directory(abs_dest)
+
             # are we at the end of a directory?
             if rel_dest.endswith('/') or rel_dest == '':
-                _, dirnames, filenames = next(os.walk(abs_dest))
-                names = dirnames + filenames
+                if directory.content_loaded:
+                    # Take the order from the directory object
+                    names = [f.basename for f in directory.files]
+                    if self.fm.thisfile.basename in names:
+                        i = names.index(self.fm.thisfile.basename)
+                        names = names[i:] + names[:i]
+                else:
+                    # Fall back to old method with "os.walk"
+                    _, dirnames, filenames = next(os.walk(abs_dest))
+                    names = dirnames + filenames
+                    names.sort()
 
             # are we in the middle of the filename?
             else:
-                _, dirnames, filenames = next(os.walk(abs_dirname))
-                names = [name for name in (dirnames + filenames) \
-                        if name.startswith(rel_basename)]
+                if directory.content_loaded:
+                    # Take the order from the directory object
+                    names = [f.basename for f in directory.files \
+                            if f.basename.startswith(rel_basename)]
+                    if self.fm.thisfile.basename in names:
+                        i = names.index(self.fm.thisfile.basename)
+                        names = names[i:] + names[:i]
+                else:
+                    # Fall back to old method with "os.walk"
+                    _, dirnames, filenames = next(os.walk(abs_dirname))
+                    names = [name for name in (dirnames + filenames) \
+                            if name.startswith(rel_basename)]
+                    names.sort()
         except (OSError, StopIteration):
             # os.walk found nothing
             pass
         else:
-            names.sort()
-
             # no results, return None
             if len(names) == 0:
                 return
@@ -342,4 +391,28 @@ class AliasCommand(Command):
     _function_name = "unknown"
     _line = ""
     def execute(self):
-        self.fm.execute_console(self._line + ' ' + self.rest(1))
+        return self._make_cmd().execute()
+
+    def quick(self):
+        return self._make_cmd().quick()
+
+    def tab(self):
+        return self._make_cmd().tab()
+
+    def cancel(self):
+        return self._make_cmd().cancel()
+
+    def _make_cmd(self):
+        cmd_class = self.fm.commands.get_command(self._line.split()[0])
+        cmd = cmd_class(self._line + ' ' + self.rest(1))
+        cmd.quickly_executed = self.quickly_executed
+        cmd.quantifier = self.quantifier
+        cmd.escape_macros_for_shell = self.escape_macros_for_shell
+        cmd.resolve_macros = self.resolve_macros
+        cmd.allow_abbrev = self.allow_abbrev
+        return cmd
+
+
+if __name__ == '__main__':
+    import doctest
+    doctest.testmod()
