@@ -1,5 +1,5 @@
-# Copyright (C) 2009-2013  Roman Zimbelmann <hut@lavabit.com>
-# This software is distributed under the terms of the GNU GPL version 3.
+# This file is part of ranger, the console file manager.
+# License: GNU GPL version 3, see the file "AUTHORS" for details.
 
 import codecs
 import os
@@ -11,6 +11,8 @@ from os.path import join, isdir, realpath, exists
 from os import link, symlink, getcwd, listdir, stat
 from inspect import cleandoc
 from stat import S_IEXEC
+from hashlib import sha1
+from sys import version_info
 
 import ranger
 from ranger.ext.direction import Direction
@@ -19,20 +21,21 @@ from ranger.ext.keybinding_parser import key_to_string, construct_keybinding
 from ranger.ext.shell_escape import shell_quote
 from ranger.ext.next_available_filename import next_available_filename
 from ranger.ext.rifle import squash_flags, ASK_COMMAND
-from ranger.core.shared import FileManagerAware, EnvironmentAware, \
-        SettingsAware
+from ranger.core.shared import FileManagerAware, SettingsAware
 from ranger.core.tab import Tab
-from ranger.fsobject import File
+from ranger.container.file import File
 from ranger.core.loader import CommandLoader, CopyLoader
-from ranger.container.settingobject import ALLOWED_SETTINGS
+from ranger.container.settings import ALLOWED_SETTINGS
+from ranger.core.linemode import DEFAULT_LINEMODE
 
 MACRO_FAIL = "<\x01\x01MACRO_HAS_NO_VALUE\x01\01>"
 
 class _MacroTemplate(string.Template):
     """A template for substituting macros in commands"""
     delimiter = ranger.MACRO_DELIMITER
+    idpattern = r"[_a-z0-9]*"
 
-class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
+class Actions(FileManagerAware, SettingsAware):
     # --------------------------
     # -- Basic Commands
     # --------------------------
@@ -48,6 +51,8 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
         self.garbage_collect(-1)
         self.enter_dir(old_path)
         self.change_mode('normal')
+        if self.metadata:
+            self.metadata.reset()
 
     def change_mode(self, mode):
         if mode == self.mode:
@@ -67,14 +72,14 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
         self.mode = mode
         self.ui.status.request_redraw()
 
-    def set_option_from_string(self, option_name, value, localpath=None):
+    def set_option_from_string(self, option_name, value, localpath=None, tags=None):
         if option_name not in ALLOWED_SETTINGS:
             raise ValueError("The option named `%s' does not exist" %
                     option_name)
         if not isinstance(value, str):
             raise ValueError("The value for an option needs to be a string.")
 
-        self.settings.set(option_name, self._parse_option_value(option_name, value), localpath)
+        self.settings.set(option_name, self._parse_option_value(option_name, value), localpath, tags)
 
 
     def _parse_option_value(self, name, value):
@@ -97,9 +102,11 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
             return value.split(',')
         raise ValueError("Invalid value `%s' for option `%s'!" % (name, value))
 
-    def toggle_visual_mode(self, reverse=False):
+    def toggle_visual_mode(self, reverse=False, narg=None):
         if self.mode == 'normal':
             self._visual_reverse = reverse
+            if narg != None:
+                self.mark_files(val=not reverse, narg=narg)
             self.change_mode('visual')
         else:
             self.change_mode('normal')
@@ -202,14 +209,15 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
         macros = {}
 
         macros['rangerdir'] = ranger.RANGERDIR
+        macros['space'] = ' '
 
         if self.fm.thisfile:
-            macros['f'] = self.fm.thisfile.basename
+            macros['f'] = self.fm.thisfile.relative_path
         else:
             macros['f'] = MACRO_FAIL
 
         if self.fm.thistab.get_selection:
-            macros['s'] = [fl.basename for fl in self.fm.thistab.get_selection()]
+            macros['s'] = [fl.relative_path for fl in self.fm.thistab.get_selection()]
         else:
             macros['s'] = MACRO_FAIL
 
@@ -219,7 +227,7 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
             macros['c'] = MACRO_FAIL
 
         if self.fm.thisdir.files:
-            macros['t'] = [fl.basename for fl in self.fm.thisdir.files
+            macros['t'] = [fl.relative_path for fl in self.fm.thisdir.files
                     if fl.realpath in (self.fm.tags or [])]
         else:
             macros['t'] = MACRO_FAIL
@@ -285,7 +293,7 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
     def source(self, filename):
         filename = os.path.expanduser(filename)
         for line in open(filename, 'r'):
-            line = line.rstrip("\r\n")
+            line = line.lstrip().rstrip("\r\n")
             if line.startswith("#") or not line.strip():
                 continue
             try:
@@ -298,12 +306,17 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
                             (line, str(e)), bad=True)
 
     def execute_file(self, files, **kw):
-        """Execute a file.
+        """Uses the "rifle" module to open/execute a file
 
-        app is the name of a method in Applications, without the "app_"
-        flags is a string consisting of runner.ALLOWED_FLAGS
-        mode is a positive integer.
-        Both flags and mode specify how the program is run."""
+        Arguments are the same as for ranger.ext.rifle.Rifle.execute:
+
+        files: a list of file objects (not strings!)
+        number: a number to select which way to open the file, in case there
+            are multiple choices
+        label: a string to select an opening method by its label
+        flags: a string specifying additional options, see `man rifle`
+        mimetyle: pass the mimetype to rifle, overriding its own guess
+        """
 
         mode = kw['mode'] if 'mode' in kw else 0
 
@@ -411,6 +424,9 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
                             cwd.mark_item(f, False)
                         for f in old - current - targets:
                             cwd.mark_item(f, True)
+                if self.ui.pager.visible:
+                    self.display_file()
+
 
     def move_parent(self, n, narg=None):
         self.change_mode('normal')
@@ -444,7 +460,15 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
     def enter_dir(self, path, remember=False, history=True):
         """Enter the directory at the given path"""
         cwd = self.thisdir
+        # csh variable is lowercase
+        cdpath = os.environ.get('CDPATH', None) or os.environ.get('cdpath', None)
         result = self.thistab.enter_dir(path, history=history)
+        if result == 0 and cdpath:
+            for p in cdpath.split(':'):
+                curpath = os.path.join(p, path)
+                if os.path.isdir(curpath):
+                    result = self.thistab.enter_dir(curpath, history=history)
+                    break
         if cwd != self.thisdir:
             if remember:
                 self.bookmarks.remember(cwd)
@@ -480,7 +504,7 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
     # --------------------------
 
     def pager_move(self, narg=None, **kw):
-        self.ui.browser.pager.move(narg=narg, **kw)
+        self.ui.get_pager().move(narg=narg, **kw)
 
     def taskview_move(self, narg=None, **kw):
         self.ui.taskview.move(narg=narg, **kw)
@@ -535,7 +559,7 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
         except:
             pass
 
-    def mark_files(self, all=False, toggle=False, val=None, movedown=None, narg=1):
+    def mark_files(self, all=False, toggle=False, val=None, movedown=None, narg=None):
         """A wrapper for the directory.mark_xyz functions.
 
         Arguments:
@@ -558,6 +582,11 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
         if val is None and toggle is False:
             return
 
+        if narg == None:
+            narg = 1
+        else:
+            all = False
+
         if all:
             if toggle:
                 cwd.toggle_all_marks()
@@ -775,11 +804,15 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
         if not self.thisfile or not self.thisfile.is_file:
             return
 
-        pager = self.ui.open_embedded_pager()
+        pager = self.ui.open_pager()
         if self.settings.preview_images and self.thisfile.image:
             pager.set_image(self.thisfile.realpath)
         else:
-            pager.set_source(self.thisfile.get_preview_source(pager.wid, pager.hei))
+            f = self.thisfile.get_preview_source(pager.wid, pager.hei)
+            if self.thisfile.is_image_preview():
+                pager.set_image(f)
+            else:
+                pager.set_source(f)
 
     # --------------------------
     # -- Previews
@@ -791,10 +824,23 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
         except:
             return False
 
+    if version_info[0] == 3:
+        def sha1_encode(self, path):
+            return os.path.join(ranger.CACHEDIR,
+                    sha1(path.encode('utf-8', 'backslashreplace')) \
+                            .hexdigest()) + '.jpg'
+    else:
+        def sha1_encode(self, path):
+            return os.path.join(ranger.CACHEDIR,
+                    sha1(path).hexdigest()) + '.jpg'
+
     def get_preview(self, file, width, height):
-        pager = self.ui.browser.pager
+        pager = self.ui.get_pager()
         path = file.realpath
 
+        if not path or not os.path.exists(path):
+            return None
+
         if self.settings.preview_images and file.image:
             pager.set_image(path)
             return None
@@ -833,8 +879,17 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
                     return None
 
                 data['loading'] = True
+
+                cacheimg = os.path.join(ranger.CACHEDIR, self.sha1_encode(path))
+                if (os.path.isfile(cacheimg) and os.path.getmtime(cacheimg) > os.path.getmtime(path)):
+                    data['foundpreview'] = True
+                    data['imagepreview'] = True
+                    pager.set_image(cacheimg)
+                    data['loading'] = False
+                    return cacheimg
+
                 loadable = CommandLoader(args=[self.settings.preview_script,
-                    path, str(width), str(height)], read=True,
+                    path, str(width), str(height), cacheimg], read=True,
                     silent=True, descr="Getting preview of %s" % path)
                 def on_after(signal):
                     exit = signal.process.poll()
@@ -848,6 +903,8 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
                         data[(width, -1)] = content
                     elif exit == 5:
                         data[(-1, -1)] = content
+                    elif exit == 6:
+                        data['imagepreview'] = True
                     elif exit == 1:
                         data[(-1, -1)] = None
                         data['foundpreview'] = False
@@ -866,10 +923,14 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
                     if self.thisfile and self.thisfile.realpath == path:
                         self.ui.browser.need_redraw = True
                     data['loading'] = False
-                    pager = self.ui.browser.pager
+                    pager = self.ui.get_pager()
                     if self.thisfile and self.thisfile.is_file:
-                        pager.set_source(self.thisfile.get_preview_source(
-                            pager.wid, pager.hei))
+                        if 'imagepreview' in data:
+                            pager.set_image(cacheimg)
+                            return cacheimg
+                        else:
+                            pager.set_source(self.thisfile.get_preview_source(
+                                pager.wid, pager.hei))
                 def on_destroy(signal):
                     try:
                         del self.previews[path]
@@ -947,7 +1008,9 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
                             new=self.thistab)
                     break
 
-    def tab_move(self, offset):
+    def tab_move(self, offset, narg=None):
+        if narg:
+            return self.tab_open(narg)
         assert isinstance(offset, int)
         tablist = self._get_tab_list()
         current_index = tablist.index(self.current_tab)
@@ -955,11 +1018,55 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
         if newtab != self.current_tab:
             self.tab_open(newtab)
 
-    def tab_new(self, path=None):
+    def tab_new(self, path=None, narg=None):
+        if narg:
+            return self.tab_open(narg, path)
         for i in range(1, 10):
             if not i in self.tabs:
-                self.tab_open(i, path)
-                break
+                return self.tab_open(i, path)
+
+    def tab_switch(self, path, create_directory=False):
+        """Switches to tab of given path, opening a new tab as necessary.
+
+        If path does not exist, it is treated as a directory.
+        """
+        path = realpath(path)
+        if not os.path.exists(path):
+            file_selection = None
+            if create_directory:
+                try:
+                    os.makedirs(path, exist_ok=True)
+                except OSError as err:
+                    self.fm.notify(err, bad=True)
+                    return
+                target_directory = path
+            else:
+                # Give benefit of the doubt.
+                potential_parent = os.path.dirname(path)
+                if os.path.exists(potential_parent) and os.path.isdir(potential_parent):
+                    target_directory = potential_parent
+                else:
+                    self.fm.notify("Unable to resolve given path.", bad=True)
+                    return
+        elif os.path.isdir(path):
+            file_selection = None
+            target_directory = path
+        else:
+            file_selection = path
+            target_directory = os.path.dirname(path)
+
+        for name in self.fm.tabs:
+            tab = self.fm.tabs[name]
+            # Is a tab already open?
+            if tab.path == target_directory:
+                self.fm.tab_open(name=name)
+                if file_selection:
+                    self.fm.select_file(file_selection)
+                return
+
+        self.fm.tab_new(path=target_directory)
+        if file_selection:
+            self.fm.select_file(file_selection)
 
     def _get_tab_list(self):
         assert len(self.tabs) > 0, "There must be >=1 tabs at all times"
@@ -1021,7 +1128,6 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
         self.run([pager, temporary_file.name])
 
     def dump_settings(self):
-        from ranger.container.settingobject import ALLOWED_SETTINGS
         temporary_file = tempfile.NamedTemporaryFile()
         def write(string):
             temporary_file.write(string.encode('utf-8'))
@@ -1143,7 +1249,7 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
 
     def mkdir(self, name):
         try:
-            os.mkdir(os.path.join(self.thisdir.path, name))
+            os.makedirs(os.path.join(self.thisdir.path, name))
         except OSError as err:
             self.notify(err)
 
@@ -1155,3 +1261,5 @@ class Actions(FileManagerAware, EnvironmentAware, SettingsAware):
             os.renames(src, dest)
         except OSError as err:
             self.notify(err)
+            return False
+        return True
