@@ -1,5 +1,5 @@
-# Copyright (C) 2009-2013  Roman Zimbelmann <hut@lavabit.com>
-# This software is distributed under the terms of the GNU GPL version 3.
+# This file is part of ranger, the console file manager.
+# License: GNU GPL version 3, see the file "AUTHORS" for details.
 
 """The main function responsible to initialize the FM object and stuff."""
 
@@ -10,6 +10,7 @@ def main():
     """initialize objects and run the filemanager"""
     import locale
     import ranger.api
+    from ranger.container.settings import Settings
     from ranger.core.shared import FileManagerAware, SettingsAware
     from ranger.core.fm import FM
 
@@ -52,7 +53,7 @@ def main():
                     sys.stdout.write(line)
         return 1 if arg.fail_unless_cd else 0 # COMPAT
 
-    SettingsAware._setup(clean=arg.clean)
+    SettingsAware._setup(Settings())
 
     if arg.selectfile:
         arg.selectfile = os.path.abspath(arg.selectfile)
@@ -87,7 +88,7 @@ def main():
     try:
         # Initialize objects
         fm = FM(paths=targets)
-        FileManagerAware.fm = fm
+        FileManagerAware._setup(fm)
         load_settings(fm, arg.clean)
 
         if arg.list_unused_keys:
@@ -109,6 +110,12 @@ def main():
             from ranger.ext import curses_interrupt_handler
             curses_interrupt_handler.install_interrupt_handler()
 
+        # Create cache directory
+        if fm.settings.preview_images and fm.settings.use_preview_script:
+            from ranger import CACHEDIR
+            if not os.path.exists(CACHEDIR):
+                os.makedirs(CACHEDIR)
+
         # Run the file manager
         fm.initialize()
         ranger.api.hook_init(fm)
@@ -158,7 +165,7 @@ def main():
             print(crash_traceback)
             print("ranger crashed.  " \
                 "Please report this traceback at:")
-            print("http://savannah.nongnu.org/bugs/?group=ranger&func=additem")
+            print("https://github.com/hut/ranger/issues")
             return 1
         return 0
 
@@ -186,7 +193,7 @@ def parse_arguments():
             help="change the configuration directory. (%default)")
     parser.add_option('--copy-config', type='string', metavar='which',
             help="copy the default configs to the local config directory. "
-            "Possible values: all, rc, rifle, commands, scope")
+            "Possible values: all, rc, rifle, commands, commands_full, scope")
     parser.add_option('--fail-unless-cd', action='store_true',
             help=SUPPRESS_HELP)  # COMPAT
     parser.add_option('-m', '--mode', type='int', default=0, metavar='n',
@@ -247,11 +254,14 @@ def load_settings(fm, clean):
 
         # Load custom commands
         if os.path.exists(fm.confpath('commands.py')):
+            old_bytecode_setting = sys.dont_write_bytecode
+            sys.dont_write_bytecode = True
             try:
                 import commands
                 fm.commands.load_commands_from_module(commands)
             except ImportError:
                 pass
+            sys.dont_write_bytecode = old_bytecode_setting
 
         allow_access_to_confdir(ranger.arg.confdir, False)
 
@@ -282,7 +292,7 @@ def load_settings(fm, clean):
             for plugin in sorted(plugins):
                 try:
                     module = __import__('plugins', fromlist=[plugin])
-                    fm.log.append("Loaded plugin '%s'." % module)
+                    fm.log.append("Loaded plugin '%s'." % plugin)
                 except Exception as e:
                     fm.log.append("Error in plugin '%s'" % plugin)
                     import traceback
@@ -294,7 +304,7 @@ def load_settings(fm, clean):
         # options.py[oc] are deliberately ignored
         if os.path.exists(fm.confpath("options.py")):
             module = __import__('options')
-            from ranger.container.settingobject import ALLOWED_SETTINGS
+            from ranger.container.settings import ALLOWED_SETTINGS
             for setting in ALLOWED_SETTINGS:
                 if hasattr(module, setting):
                     fm.settings[setting] = getattr(module, setting)
