@@ -19,7 +19,7 @@ import re
 from subprocess import Popen, PIPE
 import sys
 
-__version__ = 'rifle 1.7.0'
+__version__ = 'rifle 1.7.1'
 
 # Options and constants that a user might want to change:
 DEFAULT_PAGER = 'less'
@@ -232,6 +232,8 @@ class Rifle(object):
             return True
         elif function == 'X':
             return 'DISPLAY' in os.environ
+        elif function == 'env':
+            return bool(os.environ.get(argument))
         elif function == 'else':
             return True
 
