@@ -1,5 +1,5 @@
-# Copyright (C) 2009-2013  Roman Zimbelmann <hut@lavabit.com>
-# This software is distributed under the terms of the GNU GPL version 3.
+# This file is part of ranger, the console file manager.
+# License: GNU GPL version 3, see the file "AUTHORS" for details.
 
 """Contains abbreviations to curses color/attribute constants.
 
@@ -15,6 +15,8 @@ bool(attr & reverse) # => False
 
 import curses
 
+DEFAULT_FOREGROUND = curses.COLOR_WHITE
+DEFAULT_BACKGROUND = curses.COLOR_BLACK
 COLOR_PAIRS = {10: 0}
 
 def get_color(fg, bg):
@@ -23,7 +25,17 @@ def get_color(fg, bg):
     key = (fg, bg)
     if key not in COLOR_PAIRS:
         size = len(COLOR_PAIRS)
-        curses.init_pair(size, fg, bg)
+        try:
+            curses.init_pair(size, fg, bg)
+        except:
+            # If curses.use_default_colors() failed during the initialization
+            # of curses, then using -1 as fg or bg will fail as well, which
+            # we need to handle with fallback-defaults:
+            if fg == -1:  # -1 is the "default" color
+                fg = DEFAULT_FOREGROUND
+            if bg == -1:  # -1 is the "default" color
+                bg = DEFAULT_BACKGROUND
+            curses.init_pair(size, fg, bg)
         COLOR_PAIRS[key] = size
 
     return COLOR_PAIRS[key]
