@@ -1,5 +1,5 @@
-# Copyright (C) 2009-2013  Roman Zimbelmann <hut@lavabit.com>
-# This software is distributed under the terms of the GNU GPL version 3.
+# This file is part of ranger, the console file manager.
+# License: GNU GPL version 3, see the file "AUTHORS" for details.
 
 CONTEXT_KEYS = ['reset', 'error', 'badinfo',
         'in_browser', 'in_statusbar', 'in_titlebar', 'in_console',
@@ -11,11 +11,15 @@ CONTEXT_KEYS = ['reset', 'error', 'badinfo',
         'good', 'bad',
         'space', 'permissions', 'owner', 'group', 'mtime', 'nlink',
         'scroll', 'all', 'bot', 'top', 'percentage', 'filter',
-        'marked', 'tagged', 'tag_marker', 'cut', 'copied',
+        'flat', 'marked', 'tagged', 'tag_marker', 'cut', 'copied',
         'help_markup', # COMPAT
         'seperator', 'key', 'special', 'border', # COMPAT
         'title', 'text', 'highlight', 'bars', 'quotes', 'tab', 'loaded',
-        'keybuffer']
+        'keybuffer',
+        'infostring',
+        'vcsfile', 'vcsremote', 'vcsinfo', 'vcscommit',
+        'vcsconflict', 'vcschanged', 'vcsunknown', 'vcsignored',
+        'vcsstaged', 'vcssync', 'vcsbehind', 'vcsahead', 'vcsdiverged']
 
 class Context(object):
     def __init__(self, keys):
