@@ -1,5 +1,5 @@
-# Copyright (C) 2009-2013  Roman Zimbelmann <hut@lavabit.com>
-# This software is distributed under the terms of the GNU GPL version 3.
+# This file is part of ranger, the console file manager.
+# License: GNU GPL version 3, see the file "AUTHORS" for details.
 
 """The BrowserView manages a set of BrowserColumns."""
 
@@ -40,6 +40,8 @@ class BrowserView(Widget, DisplayableContainer):
         self.fm.signal_bind('move', self.request_clear)
         self.settings.signal_bind('setopt.column_ratios', self.request_clear)
 
+        self.old_draw_borders = self.settings.draw_borders
+
     def change_ratios(self, ratios):
         if isinstance(ratios, Signal):
             ratios = ratios.value
@@ -109,7 +111,6 @@ class BrowserView(Widget, DisplayableContainer):
                 self.fm.ui.win.move(self.main_column.y, self.main_column.x)
             except:
                 pass
-            self.pager.draw_image()
         else:
             try:
                 x = self.main_column.x
@@ -118,7 +119,6 @@ class BrowserView(Widget, DisplayableContainer):
                 self.fm.ui.win.move(y, x)
             except:
                 pass
-            self.columns[-1].draw_image()
 
     def _draw_borders(self):
         win = self.win
@@ -163,8 +163,8 @@ class BrowserView(Widget, DisplayableContainer):
             y = self.hei - 1
             try:
                 win.vline(1, x, curses.ACS_VLINE, y - 1)
-                win.addch(0, x, curses.ACS_TTEE, 0)
-                win.addch(y, x, curses.ACS_BTEE, 0)
+                self.addch(0, x, curses.ACS_TTEE, 0)
+                self.addch(y, x, curses.ACS_BTEE, 0)
             except:
                 # in case it's off the boundaries
                 pass
@@ -360,3 +360,7 @@ class BrowserView(Widget, DisplayableContainer):
                 # force clearing the image when resizing preview column
                 self.columns[-1].clear_image(force=True)
             self.resize(self.y, self.x, self.hei, self.wid)
+
+        if self.old_draw_borders != self.settings.draw_borders:
+            self.resize(self.y, self.x, self.hei, self.wid)
+            self.old_draw_borders = self.settings.draw_borders
